Package: googlesheets4
Title: Access Google Sheets using the Sheets API V4
Version: 1.1.2
Authors@R: c(
    person("Jennifer", "Bryan", , "jenny@posit.co", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0002-6983-2759")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Interact with Google Sheets through the Sheets API v4
    <https://developers.google.com/sheets/api>. "API" is an acronym for
    "application programming interface"; the Sheets API allows users to
    interact with Google Sheets programmatically, instead of via a web
    browser. The "v4" refers to the fact that the Sheets API is currently
    at version 4. This package can read and write both the metadata and
    the cell data in a Sheet.
License: MIT + file LICENSE
URL: https://googlesheets4.tidyverse.org,
        https://github.com/tidyverse/googlesheets4
BugReports: https://github.com/tidyverse/googlesheets4/issues
Depends: R (>= 3.6)
Imports: cellranger, cli (>= 3.0.0), curl, gargle (>= 1.6.0), glue (>=
        1.3.0), googledrive (>= 2.1.0), httr, ids, lifecycle, magrittr,
        methods, purrr, rematch2, rlang (>= 1.0.2), tibble (>= 2.1.1),
        utils, vctrs (>= 0.2.3), withr
Suggests: readr, rmarkdown, spelling, testthat (>= 3.1.7)
ByteCompile: true
Config/Needs/website: tidyverse, tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.2.9000
NeedsCompilation: no
Packaged: 2025-09-03 23:00:43 UTC; jenny
Author: Jennifer Bryan [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-6983-2759>),
  Posit Software, PBC [cph, fnd]
Maintainer: Jennifer Bryan <jenny@posit.co>
Repository: CRAN
Date/Publication: 2025-09-03 23:20:02 UTC
Built: R 4.6.0; ; 2026-04-28 22:49:21 UTC; unix
