# Generated by roxygen2: do not edit by hand

S3method(as_CellData,"NULL")
S3method(as_CellData,Date)
S3method(as_CellData,POSIXct)
S3method(as_CellData,character)
S3method(as_CellData,default)
S3method(as_CellData,factor)
S3method(as_CellData,googlesheets4_formula)
S3method(as_CellData,googlesheets4_schema_CellData)
S3method(as_CellData,list)
S3method(as_CellData,logical)
S3method(as_CellData,numeric)
S3method(as_GridCoordinate,default)
S3method(as_GridCoordinate,range_spec)
S3method(as_GridRange,default)
S3method(as_GridRange,range_spec)
S3method(as_NamedRange,default)
S3method(as_NamedRange,range_spec)
S3method(as_Sheet,"NULL")
S3method(as_Sheet,character)
S3method(as_Sheet,data.frame)
S3method(as_Sheet,default)
S3method(as_id,googlesheets4_spreadsheet)
S3method(as_id,sheets_id)
S3method(as_range_spec,"NULL")
S3method(as_range_spec,cell_limits)
S3method(as_range_spec,character)
S3method(as_range_spec,default)
S3method(as_sheets_id,"NULL")
S3method(as_sheets_id,character)
S3method(as_sheets_id,default)
S3method(as_sheets_id,dribble)
S3method(as_sheets_id,drive_id)
S3method(as_sheets_id,googlesheets4_spreadsheet)
S3method(as_sheets_id,sheets_id)
S3method(as_tibble,googlesheets4_schema_GridRange)
S3method(as_tibble,googlesheets4_schema_NamedRange)
S3method(as_tibble,googlesheets4_schema_ProtectedRange)
S3method(as_tibble,googlesheets4_schema_Sheet)
S3method(as_tibble,googlesheets4_schema_SheetProperties)
S3method(ctype,"NULL")
S3method(ctype,SHEETS_CELL)
S3method(ctype,character)
S3method(ctype,default)
S3method(ctype,list)
S3method(format,googlesheets4_schema_CellData)
S3method(format,googlesheets4_spreadsheet)
S3method(format,range_spec)
S3method(format,sheets_id)
S3method(patch,default)
S3method(patch,googlesheets4_schema)
S3method(print,googlesheets4_schema_CellData)
S3method(print,googlesheets4_spreadsheet)
S3method(print,range_spec)
S3method(print,sheets_id)
S3method(vec_cast,character.sheets_id)
S3method(vec_cast,drive_id.sheets_id)
S3method(vec_cast,googlesheets4_formula)
S3method(vec_cast,sheets_id.character)
S3method(vec_cast,sheets_id.drive_id)
S3method(vec_cast,sheets_id.sheets_id)
S3method(vec_cast.character,googlesheets4_formula)
S3method(vec_cast.googlesheets4_formula,character)
S3method(vec_cast.googlesheets4_formula,default)
S3method(vec_cast.googlesheets4_formula,googlesheets4_formula)
S3method(vec_ptype2,character.sheets_id)
S3method(vec_ptype2,drive_id.sheets_id)
S3method(vec_ptype2,googlesheets4_formula)
S3method(vec_ptype2,sheets_id.character)
S3method(vec_ptype2,sheets_id.drive_id)
S3method(vec_ptype2,sheets_id.sheets_id)
S3method(vec_ptype2.character,googlesheets4_formula)
S3method(vec_ptype2.googlesheets4_formula,character)
S3method(vec_ptype2.googlesheets4_formula,default)
S3method(vec_ptype2.googlesheets4_formula,googlesheets4_formula)
S3method(vec_ptype_abbr,googlesheets4_formula)
S3method(vec_ptype_abbr,sheets_id)
export("%>%")
export(anchored)
export(as_sheets_id)
export(cell_cols)
export(cell_limits)
export(cell_rows)
export(gs4_api_key)
export(gs4_auth)
export(gs4_auth_configure)
export(gs4_browse)
export(gs4_create)
export(gs4_deauth)
export(gs4_endpoints)
export(gs4_example)
export(gs4_examples)
export(gs4_find)
export(gs4_fodder)
export(gs4_formula)
export(gs4_get)
export(gs4_has_token)
export(gs4_oauth_app)
export(gs4_oauth_client)
export(gs4_random)
export(gs4_scopes)
export(gs4_token)
export(gs4_user)
export(local_gs4_quiet)
export(range_autofit)
export(range_clear)
export(range_delete)
export(range_flood)
export(range_read)
export(range_read_cells)
export(range_speedread)
export(range_write)
export(read_sheet)
export(request_generate)
export(request_make)
export(sheet_add)
export(sheet_append)
export(sheet_copy)
export(sheet_delete)
export(sheet_names)
export(sheet_properties)
export(sheet_relocate)
export(sheet_rename)
export(sheet_resize)
export(sheet_write)
export(spread_sheet)
export(vec_cast.googlesheets4_formula)
export(vec_ptype2.googlesheets4_formula)
export(with_gs4_quiet)
export(write_sheet)
import(rlang)
import(vctrs)
importFrom(cellranger,anchored)
importFrom(cellranger,cell_cols)
importFrom(cellranger,cell_limits)
importFrom(cellranger,cell_rows)
importFrom(gargle,bulletize)
importFrom(gargle,gargle_map_cli)
importFrom(glue,glue)
importFrom(glue,glue_collapse)
importFrom(glue,glue_data)
importFrom(googledrive,as_id)
importFrom(lifecycle,deprecated)
importFrom(magrittr,"%>%")
importFrom(methods,setOldClass)
importFrom(purrr,compact)
importFrom(purrr,discard)
importFrom(purrr,imap)
importFrom(purrr,keep)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map_chr)
importFrom(purrr,map_dbl)
importFrom(purrr,map_int)
importFrom(purrr,map_lgl)
importFrom(purrr,modify_if)
importFrom(purrr,pluck)
importFrom(purrr,pmap)
importFrom(purrr,pmap_chr)
importFrom(purrr,transpose)
importFrom(purrr,walk)
importFrom(tibble,as_tibble)
