This release is prompted by a request from Kurt Hornik and I am under a deadline of 2023-03-22 (I was given a few days of wiggle room due to travel). That request related to making the signature of a method match the generic.

I'm also updating my email address as maintainer.

## R CMD check results

0 errors | 0 warnings | 1 note

   New maintainer:
     Jennifer Bryan <jenny@posit.co>
   Old maintainer(s):
     Jennifer Bryan <jenny@rstudio.com>

## revdepcheck results

We checked 10 reverse dependencies, comparing R CMD check results across CRAN and dev versions of this package.

 * We saw 0 new problems
 * We failed to check 0 packages
