% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gs4_create.R
\name{gs4_random}
\alias{gs4_random}
\title{Generate a random Sheet name}
\usage{
gs4_random(n = 1)
}
\arguments{
\item{n}{Number of names to generate.}
}
\value{
A character vector.
}
\description{
Generates a random name, suitable for a newly created Sheet, using
\code{\link[ids:adjective_animal]{ids::adjective_animal()}}.
}
\examples{
gs4_random()
}
